@@ -23,7 +23,7 @@ struct LatencyProbe {
 
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     env_logger::init();
-    let mut ditto = Ditto::builder()
+    let ditto = Ditto::builder()
         .with_temp_dir()
         .with_minimum_log_level(CLogLevel::Info)
         .with_identity(|ditto_root| {
