@@ -1,7 +1,9 @@
 use_prelude!();
 pub mod builder;
 
-use std::{env, sync::RwLock};
+use std::{env, mem::ManuallyDrop, sync::RwLock};
+
+use crossbeam_utils::atomic::AtomicCell;
 
 use ffi_sdk::BoxedDitto;
 use uuid::Uuid;
@@ -22,38 +24,75 @@ use crate::{
 /// This struct is generally a handle and interface to ditto-functionality
 /// which operates in background threads.
 pub struct Ditto {
+    // Note, we `get_mut.unwrap()` this on `Drop` so make sure to only extract
+    // weak references out of this handle.
+    //
+    // We use a `ManuallyDrop` to get owned access to the `Arc` in `Ditto`'s
+    // `Drop` glue (through `ManuallyDrop::take()`).
+    // FIXME(Daniel): have a macro do this to reduce the chance of human error.
+    fields: ManuallyDrop<Arc<DittoFields>>,
+}
+
+impl std::ops::Deref for Ditto {
+    type Target = DittoFields;
+
+    #[inline]
+    fn deref(&'_ self) -> &'_ DittoFields {
+        &*self.fields
+    }
+}
+
+pub struct DittoFields {
+    // FIXME: (Ham & Daniel) - ideally we'd use this in the same way as we do
+    // with the `fields` on `Ditto` (only ever extracting weak references)
     ditto: Arc<ffi_sdk::BoxedDitto>,
     ditto_root: Arc<dyn DittoRoot>, // Arc since Identity will need a copy
     auth: Option<DittoAuthenticator>,
     #[allow(dead_code)]
-    validity_listener: Option<Arc<ValidityListener>>, // may not need to hand on to this
+    validity_listener: Option<Arc<ValidityListener>>, // may not need to hang on to this
     store: Store,
-    activated: bool,
+    activated: AtomicCell<bool>,
     site_id: SiteId,
     transports: Arc<RwLock<Transports>>,
     presence_manager_v2: Arc<PresenceManagerV2Context>,
 }
 
-impl Drop for Ditto {
-    fn drop(&mut self) {
-        // stop all transports
-        self.stop_sync();
-        // stop all servers and advertisers
-        // the idea is to maximize the chance we
-        // can get an exclusive lock
-        unsafe { ffi_sdk::ditto_shutdown(&self.ditto) };
+impl Ditto {
+    fn drop_fields(fields: Arc<DittoFields>) {
+        // We use this pattern to ensure `self` is not used
+        // after `ManuallyDrop::take()`-ing its fields.
+        impl Drop for Ditto {
+            fn drop(&mut self) {
+                // stop all transports
+                self.stop_sync();
+                // stop all servers and advertisers
+                // the idea is to maximize the chance we
+                // can get an exclusive lock
+                unsafe {
+                    ffi_sdk::ditto_shutdown(&self.ditto);
+                }
+                unsafe {
+                    Ditto::drop_fields(ManuallyDrop::take(&mut self.fields));
+                }
+            }
+        }
         ::log::debug!("Dropping Ditto instance");
-        // Now with everything shut down try to get a mut reference
-        // The most likely remaining references are the LiveQueries
-        if let Some(_witness_of_unicity) = Arc::get_mut(&mut self.ditto) {
-            // Our own strongs counts are equal to 1;
+
+        // Now with everything shut down try to get a unique access (`mut` or
+        // owned). The most likely remaining references are the LiveQueries.
+        let mut fields = Arc::try_unwrap(fields)
+            .ok()
+            .expect("outstanding strong back reference to Ditto");
+
+        if let Some(_witness_of_unicity) = Arc::get_mut(&mut fields.ditto) {
+            // Our own strong counts are equal to 1;
             // so we implicitly drop the `BoxedDitto` which will call
             // `ditto_drop(…)`
         } else {
             // We do NOT want to drop the referent (ffi::Ditto)
-            // as LiveQueres are still running. The user is responsible for
+            // as LiveQueries are still running. The user is responsible for
             // cleaning these LQs up.
-            let count = Arc::strong_count(&self.ditto);
+            let count = Arc::strong_count(&fields.ditto);
             ::log::debug!("Attempting to drop Ditto with {} live references", count);
         }
     }
@@ -71,7 +110,7 @@ impl Ditto {
     /// Start syncing on all configured transports
     pub fn try_start_sync(&self) -> Result<(), DittoError> {
         // the License must be active or else sync will immediately fail
-        if self.activated.not() {
+        if self.activated.load().not() {
             return Err(ErrorKind::NotActivated.into());
         }
 
@@ -153,7 +192,7 @@ impl Ditto {
 impl Ditto {
     /// Activate a Ditto instance by setting a license token. You cannot
     /// sync with Ditto before you have activated it.
-    pub fn set_license_token(&mut self, license_token: &str) -> Result<(), DittoError> {
+    pub fn set_license_token(&self, license_token: &str) -> Result<(), DittoError> {
         use ::safer_ffi::prelude::{AsOut, ManuallyDropMut};
         use ffi_sdk::LicenseVerificationResult;
         let c_license: char_p::Box = char_p::new(license_token);
@@ -163,11 +202,11 @@ impl Ditto {
         let res = unsafe { ffi_sdk::verify_license(c_license.as_ref(), Some(out_err_msg)) };
 
         if res == LicenseVerificationResult::LicenseOk {
-            self.activated = true;
+            self.activated.store(true);
             return Ok(());
         }
 
-        self.activated = false;
+        self.activated.store(false);
         let err_msg = err_msg.unwrap();
 
         ::log::error!("{}", err_msg);
@@ -194,13 +233,13 @@ impl Ditto {
 
     /// Activate a Ditto instance by setting a license token. You cannot
     /// sync with Ditto before you have activated it.
-    pub fn set_access_license(&mut self, license_str: &str) {
+    pub fn set_access_license(&self, license_str: &str) {
         self.set_license_token(license_str)
             .expect("Invalid Ditto License provided")
     }
 
     /// Look for a license token from a given environment variable
-    pub fn set_license_from_env(&mut self, var_name: &str) -> Result<(), DittoError> {
+    pub fn set_license_from_env(&self, var_name: &str) -> Result<(), DittoError> {
         match env::var(var_name) {
             Ok(token) => self.set_license_token(&token),
             Err(env::VarError::NotPresent) => {
@@ -255,7 +294,7 @@ impl Ditto {
         H: Fn(V2Presence) + Send + Sync + 'static,
     {
         self.presence_manager_v2
-            .add_observer(self.ditto.clone(), handler)
+            .add_observer(self.ditto.retain(), handler)
     }
 
     pub fn root_dir(&self) -> &Path {
@@ -271,7 +310,7 @@ impl Ditto {
     }
     /// Has this Ditto instance been activated yet with a valid license token
     pub fn is_activated(&self) -> bool {
-        self.activated
+        self.activated.load()
     }
 }
 
@@ -297,6 +336,12 @@ impl Ditto {
             .build()
             .expect("Failed to build Ditto Instance")
     }
+
+    pub(crate) fn new_with_fields(fields: Arc<DittoFields>) -> Ditto {
+        Ditto {
+            fields: ManuallyDrop::new(fields),
+        }
+    }
 }
 
 impl Ditto {
