@@ -56,15 +56,44 @@ pub trait Identity: private::Sealed {
 /// The only required configuration is the application's UUID, which can be
 /// found on the Ditto portal where the app is registered.
 ///
-/// By default cloud sync is enabled.
-/// This means the SDK will sync to a Big Peer in Ditto's cloud when an internet
-/// connection is available. This is controlled by the `enableDittoCloudSync`
-/// parameter. If `true` (default), a suitable wss:// URL will be added to the
-/// `TransportConfig`.
+/// By default cloud sync is enabled. This means the SDK will sync to a Big Peer
+/// in Ditto's cloud when an internet connection is available. This is
+/// controlled by the `enableDittoCloudSync` parameter. If `true` (default), a
+/// suitable wss:// URL will be added to the `TransportConfig`.
 ///
 /// To prevent cloud sync, or to specify your own URL later, pass `false`.
 ///
-/// Authentication requests are handled by Ditto's cloud by default, configured in the portal at `<https://portal.ditto.live>`.
+/// Authentication requests are handled by Ditto's cloud by default, configured
+/// in the portal at `<https://portal.ditto.live>`.
+///
+/// To use a different or on-premises authentication service, pass a custom
+/// HTTPS base URL as the * `auth_url` parameter.
+pub struct OnlineWithAuthentication {
+    ditto_root: Arc<dyn DittoRoot>,
+    auth: DittoAuthenticator,
+    app_id: AppId,
+    enable_cloud_sync: bool,
+    auth_url: String,
+}
+
+/// Deprecated - use `OnlineWithAuthentication` instead.
+///
+/// Run Ditto in secure production mode, logging on to Ditto Cloud or an
+/// on-premises authentication server. User permissions are centrally managed.
+/// Sync will not work until a  successful login has occurred.
+///
+/// The only required configuration is the application's UUID, which can be
+/// found on the Ditto portal where the app is registered.
+///
+/// By default cloud sync is enabled. This means the SDK will sync to a Big Peer
+/// in Ditto's cloud when an internet connection is available. This is
+/// controlled by the `enableDittoCloudSync` parameter. If `true` (default), a
+/// suitable wss:// URL will be added to the `TransportConfig`.
+///
+/// To prevent cloud sync, or to specify your own URL later, pass `false`.
+///
+/// Authentication requests are handled by Ditto's cloud by default, configured
+/// in the portal at `<https://portal.ditto.live>`.
 ///
 /// To use a different or on-premises authentication service, pass a custom
 /// HTTPS base URL as the * `auth_url` parameter.
@@ -104,6 +133,8 @@ pub struct Manual {
     auth: DittoAuthenticator,
 }
 
+/// Deprecated - use `OfflinePlayground` instead.
+///
 /// Develop peer-to-peer apps with no cloud connection. This mode offers no
 /// security and must only be used for development. In this mode, any string can
 /// be used as the name of the app.
@@ -113,6 +144,15 @@ pub struct Development {
     auth: DittoAuthenticator,
 }
 
+/// Develop peer-to-peer apps with no cloud connection. This mode offers no
+/// security and must only be used for development. In this mode, any string can
+/// be used as the name of the app.
+pub struct OfflinePlayground {
+    ditto_root: Arc<dyn DittoRoot>,
+    app_id: AppId,
+    auth: DittoAuthenticator,
+}
+
 impl Online {
     /// Construct an Online Identity
     /// * `ditto_root` - The directory containing ditto local storage
@@ -156,7 +196,9 @@ impl Online {
         };
 
         let authenticator = DittoAuthenticator::new(Arc::new(auth_client));
-        ctx.lock().unwrap().set_authenticator(authenticator.clone());
+        ctx.lock()
+            .unwrap()
+            .set_authenticator(authenticator.retain());
 
         Ok(Online {
             app_id,
@@ -182,7 +224,7 @@ impl Identity for Online {
     }
 
     fn ditto_root(&self) -> Arc<dyn DittoRoot> {
-        self.ditto_root.clone()
+        self.ditto_root.retain()
     }
 
     fn is_web_valid(&self) -> bool {
@@ -206,7 +248,110 @@ impl Identity for Online {
     }
 
     fn authenticator(&self) -> Option<DittoAuthenticator> {
-        Some(self.auth.clone())
+        Some(self.auth.retain())
+    }
+
+    fn make_listener(&self, transports: Arc<RwLock<Transports>>) -> Option<Arc<ValidityListener>> {
+        Some(ValidityListener::new(transports, self.auth.auth_client()))
+    }
+}
+
+impl OnlineWithAuthentication {
+    /// Construct an OnlineWithAuthentication Identity
+    /// * `ditto_root` - The directory containing ditto local storage
+    /// * `app_id` - The unique identifier of the application, usually a UUIDv4
+    ///   string
+    /// * `auth_event_handler` - A user created type which implements the
+    ///   `DittoAuthenticationEventHandler` trait
+    /// * `enable_cloud_sync` - where cloud sync should be started by default
+    /// * `custom_auth_url` - an optional custom authentication URL; default is
+    ///   is `cloud.ditto.live`
+    pub fn new(
+        ditto_root: Arc<dyn DittoRoot>,
+        app_id: AppId,
+        auth_event_handler: impl DittoAuthenticationEventHandler + 'static,
+        enable_cloud_sync: bool,
+        custom_auth_url: Option<&str>,
+    ) -> Result<Self, DittoError> {
+        let auth_url = match custom_auth_url {
+            Some(url) => url.to_owned(),
+            None => app_id.default_auth_url(),
+        };
+        let c_working_dir = ditto_root.data_dir_to_c_str()?;
+        let c_app_id = char_p::new(app_id.to_string());
+        let c_auth_url = char_p::new(auth_url.as_str());
+
+        let login_provider = LoginProvider::new(auth_event_handler);
+
+        let LoginProvider {
+            _provider: c_provider,
+            ctx,
+        } = login_provider;
+
+        let auth_client = unsafe {
+            ffi_sdk::ditto_auth_client_make_with_web(
+                c_working_dir.as_ref(),
+                c_app_id.as_ref(),
+                c_auth_url.as_ref(),
+                Some(c_provider),
+            )
+            .ok_or(ErrorKind::Config)?
+        };
+
+        let authenticator = DittoAuthenticator::new(Arc::new(auth_client));
+        ctx.lock()
+            .unwrap()
+            .set_authenticator(authenticator.retain());
+
+        Ok(OnlineWithAuthentication {
+            app_id,
+            ditto_root,
+            auth: authenticator,
+            enable_cloud_sync,
+            auth_url,
+        })
+    }
+}
+
+impl Identity for OnlineWithAuthentication {
+    fn site_id(&self) -> SiteId {
+        unsafe { ffi_sdk::ditto_auth_client_get_site_id(&self.auth_client()) }
+    }
+
+    fn app_id(&self) -> Option<&AppId> {
+        Some(&self.app_id)
+    }
+
+    fn auth_client(&self) -> Arc<BoxedAuthClient> {
+        self.auth.auth_client()
+    }
+
+    fn ditto_root(&self) -> Arc<dyn DittoRoot> {
+        self.ditto_root.retain()
+    }
+
+    fn is_web_valid(&self) -> bool {
+        unsafe { ffi_sdk::ditto_auth_client_is_web_valid(&self.auth_client()) != 0 }
+    }
+
+    fn is_x509_valid(&self) -> bool {
+        unsafe { ffi_sdk::ditto_auth_client_is_x509_valid(&self.auth_client()) != 0 }
+    }
+
+    fn is_cloud_sync_enabled(&self) -> bool {
+        self.enable_cloud_sync
+    }
+
+    fn auth_url(&self) -> Result<String, DittoError> {
+        Ok(self.auth_url.to_owned())
+    }
+
+    fn sync_url(&self) -> Result<String, DittoError> {
+        Ok(self.app_id.default_sync_url())
+    }
+
+    fn authenticator(&self) -> Option<DittoAuthenticator> {
+        Some(self.auth.retain())
     }
 
     fn make_listener(&self, transports: Arc<RwLock<Transports>>) -> Option<Arc<ValidityListener>> {
@@ -259,7 +404,7 @@ impl Identity for Development {
     }
 
     fn ditto_root(&self) -> Arc<dyn DittoRoot> {
-        self.ditto_root.clone()
+        self.ditto_root.retain()
     }
 
     fn is_web_valid(&self) -> bool {
@@ -288,7 +433,89 @@ impl Identity for Development {
     }
 
     fn authenticator(&self) -> Option<DittoAuthenticator> {
-        Some(self.auth.clone())
+        Some(self.auth.retain())
+    }
+
+    fn make_listener(&self, transports: Arc<RwLock<Transports>>) -> Option<Arc<ValidityListener>> {
+        Some(ValidityListener::new(transports, self.auth.auth_client()))
+    }
+}
+
+impl OfflinePlayground {
+    /// Construct an `OfflinePlayground` Identity
+    /// * `ditto_root` - Location for the local database
+    /// * `app_id` - A unique string identifying the App. For an
+    ///   OfflinePlayground Identity this does not need to be a UUIDv4.
+    pub fn new(ditto_root: Arc<dyn DittoRoot>, app_id: AppId) -> Result<Self, DittoError> {
+        let c_data_dir = ditto_root.data_dir_to_c_str()?;
+        let c_app_id = app_id.to_c_string();
+        let auth_client = unsafe {
+            ffi_sdk::ditto_auth_client_make_for_development(
+                Some(c_data_dir.as_ref()),
+                c_app_id.as_ref(),
+                0,
+            )
+            .ok_or(ErrorKind::Config)?
+        };
+        let authenticator = DittoAuthenticator::new(Arc::new(auth_client));
+        Ok(Self {
+            ditto_root,
+            app_id,
+            auth: authenticator,
+        })
+    }
+
+    /// Generate a OfflinePlayground identity with a random AppId for testing
+    pub fn random(ditto_root: Arc<dyn DittoRoot>) -> Result<Self, DittoError> {
+        let app_id = AppId::generate();
+        Self::new(ditto_root, app_id)
+    }
+}
+
+impl Identity for OfflinePlayground {
+    fn site_id(&self) -> SiteId {
+        unsafe { ffi_sdk::ditto_auth_client_get_site_id(&self.auth_client()) }
+    }
+
+    fn auth_client(&self) -> Arc<BoxedAuthClient> {
+        self.auth.auth_client()
+    }
+
+    fn app_id(&self) -> Option<&AppId> {
+        Some(&self.app_id)
+    }
+
+    fn ditto_root(&self) -> Arc<dyn DittoRoot> {
+        self.ditto_root.retain()
+    }
+
+    fn is_web_valid(&self) -> bool {
+        unsafe { ffi_sdk::ditto_auth_client_is_web_valid(&self.auth_client()) != 0 }
+    }
+
+    fn is_x509_valid(&self) -> bool {
+        unsafe { ffi_sdk::ditto_auth_client_is_x509_valid(&self.auth_client()) != 0 }
+    }
+
+    fn is_cloud_sync_enabled(&self) -> bool {
+        false
+    }
+
+    fn auth_url(&self) -> Result<String, DittoError> {
+        Err(DittoError::new(
+            ErrorKind::InvalidInput,
+            "Cloud Auth is not enabled for OfflinePlayground Identities".to_string(),
+        ))
+    }
+    fn sync_url(&self) -> Result<String, DittoError> {
+        Err(DittoError::new(
+            ErrorKind::InvalidInput,
+            "Cloud sync is not enabled for OfflinePlayground Identities".to_string(),
+        ))
+    }
+
+    fn authenticator(&self) -> Option<DittoAuthenticator> {
+        Some(self.auth.retain())
     }
 
     fn make_listener(&self, transports: Arc<RwLock<Transports>>) -> Option<Arc<ValidityListener>> {
@@ -343,7 +570,7 @@ impl Identity for SharedKey {
     }
 
     fn ditto_root(&self) -> Arc<dyn DittoRoot> {
-        self.ditto_root.clone()
+        self.ditto_root.retain()
     }
 
     fn is_web_valid(&self) -> bool {
@@ -416,7 +643,7 @@ impl Identity for Manual {
     }
 
     fn ditto_root(&self) -> Arc<dyn DittoRoot> {
-        self.ditto_root.clone()
+        self.ditto_root.retain()
     }
 
     fn is_web_valid(&self) -> bool {
@@ -510,7 +737,7 @@ impl Identity for OnlinePlayground {
     }
 
     fn ditto_root(&self) -> Arc<dyn DittoRoot> {
-        self.ditto_root.clone()
+        self.ditto_root.retain()
     }
 
     fn is_web_valid(&self) -> bool {
@@ -530,7 +757,7 @@ impl Identity for OnlinePlayground {
     }
 
     fn authenticator(&self) -> Option<DittoAuthenticator> {
-        Some(self.auth.clone())
+        Some(self.auth.retain())
     }
 
     fn make_listener(&self, transports: Arc<RwLock<Transports>>) -> Option<Arc<ValidityListener>> {
@@ -548,6 +775,8 @@ mod private {
     impl Sealed for OnlinePlayground {}
     impl Sealed for Manual {}
     impl Sealed for Online {}
+    impl Sealed for OnlineWithAuthentication {}
     impl Sealed for Development {}
+    impl Sealed for OfflinePlayground {}
     impl Sealed for SharedKey {}
 }
