@@ -3,7 +3,10 @@ pub use crate::{
     ditto::{builder::DittoBuilder, AppId, Ditto, SiteId},
     ffi_sdk::{BoxedDitto, BoxedDocument, CLogLevel},
     fs::{DittoRoot, PersistentRoot, TempRoot},
-    identity::{self, Development, Manual, Online, OnlinePlayground, SharedKey},
+    identity::{
+        self, Development, Manual, OfflinePlayground, Online, OnlinePlayground,
+        OnlineWithAuthentication, SharedKey,
+    },
     store::{
         collection::{
             document::{DittoDocument, DittoMutDocument},
@@ -17,5 +20,7 @@ pub use crate::{
         timeseries::TimeSeries,
         Store, WriteStrategy,
     },
-    transport::{HttpListenConfig, TcpListenConfig, TransportConfig},
+    transport::{
+        peers_observer::PeersObserver, HttpListenConfig, TcpListenConfig, TransportConfig,
+    },
 };
