@@ -207,7 +207,7 @@ impl Collection {
                 attachment.id.into(),
                 attachment.len,
                 metadata,
-                self.ditto.clone(),
+                self.ditto.retain(),
                 attachment.handle,
             );
             Ok(ret)
@@ -224,6 +224,6 @@ impl Collection {
         attachment_token: DittoAttachmentToken,
         on_fetch_event: impl Fn(DittoAttachmentFetchEvent) + Send + Sync + 'a,
     ) -> Result<DittoAttachmentFetcher<'a>, DittoError> {
-        DittoAttachmentFetcher::new(attachment_token, self.ditto.clone(), on_fetch_event)
+        DittoAttachmentFetcher::new(attachment_token, self.ditto.retain(), on_fetch_event)
     }
 }
