@@ -8,6 +8,8 @@ use crate::{
     utils::prelude::*,
 };
 
+const LMDB_ERROR_NOT_FOUND_CODE: i32 = -30798;
+
 /// These objects are returned when using findByID functionality on
 /// DittoCollections. You can either call exec on the object to get an immediate
 /// return value, or you can establish either a live query or a subscription,
@@ -103,13 +105,10 @@ impl PendingIdSpecificOperation {
                 &mut txn,
             )
         };
-        if result.status_code == 0 {
-            result.ok_value.ok_or_else(|| {
-                // Client Error - Document not found
-                DittoError::from_ffi(ErrorKind::NonExtant)
-            })
+        if result.status_code == LMDB_ERROR_NOT_FOUND_CODE {
+            Err(DittoError::from_ffi(ErrorKind::NonExtant))
         } else {
-            Err(DittoError::from_ffi(ErrorKind::Internal))
+            result.ok()
         }
     }
 
@@ -277,3 +276,61 @@ impl PendingIdSpecificOperation {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+
+    use serde_json::json;
+
+    use crate::{error::DittoError, prelude::*, utils::prelude::ErrorKind};
+
+    fn setup_ditto() -> Result<Ditto, DittoError> {
+        let ditto = Ditto::builder()
+            .with_temp_dir()
+            .with_identity(|ditto_root| identity::OfflinePlayground::random(ditto_root))?
+            .with_minimum_log_level(CLogLevel::Info)
+            .build()?;
+        ditto.set_license_from_env("DITTO_LICENSE")?;
+        Ok(ditto)
+    }
+
+    #[test]
+    fn test_doc_not_found() {
+        let ditto = setup_ditto().unwrap();
+        let store = ditto.store();
+        let collection = store.collection("test").unwrap();
+        let doc_id = DocumentId::new(&"test_key").unwrap();
+        let doc = collection.find_by_id(doc_id).exec();
+
+        assert!(doc.is_err());
+
+        let e = doc.err().unwrap();
+        let error_kind = e.kind();
+        assert_eq!(error_kind, ErrorKind::NonExtant);
+    }
+
+    #[test]
+    fn test_doc_found() {
+        let ditto = setup_ditto().unwrap();
+        let store = ditto.store();
+        let collection = store.collection("test").unwrap();
+        let doc_id = DocumentId::new(&"test_key").unwrap();
+        let doc_id_1 = doc_id.clone();
+        let doc = collection.find_by_id(doc_id).exec();
+
+        assert!(doc.is_err());
+
+        let e = doc.err().unwrap();
+        let error_kind = e.kind();
+        assert_eq!(error_kind, ErrorKind::NonExtant);
+
+        let content = json!({"hello": "again"});
+        let inserted = collection.insert(content, Some(&doc_id_1), false);
+
+        assert!(inserted.is_ok());
+
+        let doc = collection.find_by_id(doc_id_1).exec();
+
+        assert!(doc.is_ok());
+    }
+}
