@@ -151,6 +151,11 @@ pub struct V2Peer {
     /// The operating system of a peer (if known).
     pub os: Option<V2Os>,
 
+    /// An optional mesh role which an app developer might provide
+    /// to influence connection priorities. Values can range between
+    /// 0-63 (inclusive). Defaults to `0` if not set.
+    pub mesh_role: u8,
+
     /// Flag which indicates if this peer is connected to HyDRA. This is
     /// represented as a simple flag since attempting add HyDRA as a node
     /// to a graph would be extremely convoluted. The presence viewer
